@@ -28,7 +28,7 @@ from gevent.pywsgi import WSGIServer
 
 from flask import (
     request, jsonify, make_response, Response, Response as HttpResponse,
-    send_file, session, redirect
+    send_file, session, redirect, g
 )
 from flask_api import status
 from types import SimpleNamespace
@@ -44,8 +44,9 @@ from label_studio.utils.functions import (
 from label_studio.utils.misc import (
     exception_treatment, exception_treatment_page,
     config_line_stripped, get_config_templates, convert_string_to_hash, serialize_class,
-    DirectionSwitch, check_port_in_use
+    DirectionSwitch, check_port_in_use, timestamp_to_local_datetime
 )
+from label_studio.utils.analytics import Analytics
 from label_studio.utils.argparser import parse_input_args
 from label_studio.utils.uri_resolver import resolve_task_data_uri
 from label_studio.storage import get_storage_form
@@ -121,9 +122,21 @@ def json_filter(s):
     return json.dumps(s)
 
 
-@app.before_first_request
-def app_init():
-    pass
+@app.before_request
+def app_before_request_callback():
+    # setup session cookie
+    if 'session_id' not in session:
+        session['session_id'] = str(uuid4())
+    g.project = project_get_or_create()
+    g.analytics = Analytics(input_args, g.project)
+    g.sid = g.analytics.server_id
+
+
+@app.after_request
+def app_after_request_callback(response):
+    if hasattr(g, 'analytics'):
+        g.analytics.send(request, session, response)
+    return response
 
 
 @app.route('/static/media/<path:path>')
@@ -142,8 +155,7 @@ def send_upload(path):
     """
     logger.warning('Task path starting with "/upload/" is deprecated and will be removed in next releases, '
                    'replace "/upload/" => "/data/upload/" in your tasks.json files')
-    project = project_get_or_create()
-    project_dir = os.path.join(project.name, 'upload')
+    project_dir = os.path.join(g.project.path, 'upload')
     return open(os.path.join(project_dir, path), 'rb').read()
 
 
@@ -168,8 +180,7 @@ def validation_error_handler(error):
 def labeling_page():
     """ Label studio frontend: task labeling
     """
-    project = project_get_or_create()
-    if project.no_tasks():
+    if g.project.no_tasks():
         return redirect('welcome')
 
     # task data: load task or task with completions if it exists
@@ -179,18 +190,17 @@ def labeling_page():
     if task_id is not None:
         task_id = int(task_id)
         # Task explore mode
-        task_data = project.get_task_with_completions(task_id) or project.source_storage.get(task_id)
+        task_data = g.project.get_task_with_completions(task_id) or g.project.source_storage.get(task_id)
         task_data = resolve_task_data_uri(task_data)
 
-        if project.ml_backends_connected:
-            task_data = project.make_predictions(task_data)
+        if g.project.ml_backends_connected:
+            task_data = g.project.make_predictions(task_data)
 
-    project.analytics.send(getframeinfo(currentframe()).function)
     return flask.render_template(
         'labeling.html',
-        project=project,
-        config=project.config,
-        label_config_line=project.label_config_line,
+        project=g.project,
+        config=g.project.config,
+        label_config_line=g.project.label_config_line,
         task_id=task_id,
         task_data=task_data,
         **find_editor_files()
@@ -203,14 +213,12 @@ def labeling_page():
 def welcome_page():
     """ Label studio frontend: task labeling
     """
-    project = project_get_or_create()
-    project.analytics.send(getframeinfo(currentframe()).function)
-    project.update_on_boarding_state()
+    g.project.update_on_boarding_state()
     return flask.render_template(
         'welcome.html',
-        config=project.config,
-        project=project,
-        on_boarding=project.on_boarding
+        config=g.project.config,
+        project=g.project,
+        on_boarding=g.project.on_boarding
     )
 
 
@@ -220,14 +228,12 @@ def welcome_page():
 def tasks_page():
     """ Tasks and completions page
     """
-    project = project_get_or_create()
-    serialized_project = project.serialize()
+    serialized_project = g.project.serialize()
     serialized_project['multi_session_mode'] = input_args.command != 'start-multi-session'
-    project.analytics.send(getframeinfo(currentframe()).function)
     return flask.render_template(
         'tasks.html',
-        config=project.config,
-        project=project,
+        config=g.project.config,
+        project=g.project,
         serialized_project=serialized_project,
         **find_editor_files()
     )
@@ -239,16 +245,13 @@ def tasks_page():
 def setup_page():
     """ Setup label config
     """
-    project = project_get_or_create()
-
-    templates = get_config_templates()
+    templates = get_config_templates(g.project.config)
     input_values = {}
-    project.analytics.send(getframeinfo(currentframe()).function)
     return flask.render_template(
         'setup.html',
-        config=project.config,
-        project=project,
-        label_config_full=project.label_config_full,
+        config=g.project.config,
+        project=g.project,
+        label_config_full=g.project.label_config_full,
         templates=templates,
         input_values=input_values,
         multi_session=input_args.command == 'start-multi-session'
@@ -261,13 +264,10 @@ def setup_page():
 def import_page():
     """ Import tasks from JSON, CSV, ZIP and more
     """
-    project = project_get_or_create()
-
-    project.analytics.send(getframeinfo(currentframe()).function)
     return flask.render_template(
         'import.html',
-        config=project.config,
-        project=project
+        config=g.project.config,
+        project=g.project
     )
 
 
@@ -277,13 +277,11 @@ def import_page():
 def export_page():
     """ Export completions as JSON or using converters
     """
-    project = project_get_or_create()
-    project.analytics.send(getframeinfo(currentframe()).function)
     return flask.render_template(
         'export.html',
-        config=project.config,
-        formats=project.converter.supported_formats,
-        project=project
+        config=g.project.config,
+        formats=g.project.converter.supported_formats,
+        project=g.project
     )
 
 
@@ -292,27 +290,32 @@ def export_page():
 @exception_treatment_page
 def model_page():
     """ Machine learning"""
-    project = project_get_or_create()
-    project.analytics.send(getframeinfo(currentframe()).function)
     ml_backends = []
-    for ml_backend in project.ml_backends:
+    for ml_backend in g.project.ml_backends:
         if ml_backend.connected:
             try:
-                ml_backend.sync(project)
-                training_status = ml_backend.is_training(project)
+                ml_backend.sync(g.project)
+                training_status = ml_backend.is_training(g.project)
                 ml_backend.training_in_progress = training_status['is_training']
                 ml_backend.model_version = training_status['model_version']
                 ml_backend.is_connected = True
+                ml_backend.is_error = False
             except Exception as exc:
                 logger.error(str(exc), exc_info=True)
                 ml_backend.is_error = True
+                try:
+                    # try to parse json as the result of @exception_treatment
+                    ml_backend.error = json.loads(str(exc))
+                except ValueError:
+                    ml_backend.error = {'detail': "Can't parse exception message from ML Backend"}
+
         else:
             ml_backend.is_connected = False
         ml_backends.append(ml_backend)
     return flask.render_template(
         'model.html',
-        config=project.config,
-        project=project,
+        config=g.project.config,
+        project=g.project,
         ml_backends=ml_backends
     )
 
@@ -322,9 +325,6 @@ def model_page():
 def api_render_label_studio():
     """ Label studio frontend rendering for iframe
     """
-    # get args
-    project = project_get_or_create()
-
     config = request.args.get('config', request.form.get('config', ''))
     config = unquote(config)
     if not config:
@@ -335,7 +335,7 @@ def api_render_label_studio():
     example_task_data = {
         'id': 1764,
         'data': task_data,
-        'project': project.id,
+        'project': g.project.id,
         'created_at': '2019-02-06T14:06:42.000420Z',
         'updated_at': '2019-02-06T14:06:42.000420Z'
     }
@@ -348,7 +348,6 @@ def api_render_label_studio():
     }
     response.update(find_editor_files())
 
-    project.analytics.send(getframeinfo(currentframe()).function)
     return flask.render_template('render_ls.html', **response)
 
 
@@ -359,9 +358,8 @@ def api_validate_config():
     """
     if 'label_config' not in request.form:
         return make_response('No label_config in POST', status.HTTP_417_EXPECTATION_FAILED)
-    project = project_get_or_create()
     try:
-        project.validate_label_config(request.form['label_config'])
+        g.project.validate_label_config(request.form['label_config'])
     except ValidationError as e:
         return make_response(jsonify({'label_config': e.msg_to_list()}), status.HTTP_400_BAD_REQUEST)
     except Exception as e:
@@ -381,21 +379,19 @@ def api_save_config():
     elif 'label_config' in request.json:
         label_config = request.json['label_config']
 
-    project = project_get_or_create()
     # check config before save
     try:
-        project.validate_label_config(label_config)
+        g.project.validate_label_config(label_config)
     except ValidationError as e:
         return make_response(jsonify({'label_config': e.msg_to_list()}), status.HTTP_400_BAD_REQUEST)
     except Exception as e:
         return make_response(jsonify({'label_config': [str(e)]}), status.HTTP_400_BAD_REQUEST)
 
     try:
-        project.update_label_config(label_config)
+        g.project.update_label_config(label_config)
     except Exception as e:
         return make_response(jsonify({'label_config': [str(e)]}), status.HTTP_400_BAD_REQUEST)
 
-    project.analytics.send(getframeinfo(currentframe()).function)
     return Response(status=status.HTTP_201_CREATED)
 
 
@@ -407,12 +403,11 @@ def api_import_example():
     # django compatibility
     request.GET = request.args
     request.POST = request.form
-    project = project_get_or_create()
     config = request.GET.get('label_config', '')
     if not config:
         config = request.POST.get('label_config', '')
     try:
-        project.validate_label_config(config)
+        g.project.validate_label_config(config)
         output = generate_sample_task_without_check(config, mode='editor_preview')
     except (ValueError, ValidationError, lxml.etree.Error, KeyError):
         response = HttpResponse('error while example generating', status=status.HTTP_400_BAD_REQUEST)
@@ -426,13 +421,12 @@ def api_import_example():
 def api_import_example_file():
     """ Task examples for import
     """
-    project = project_get_or_create()
     request.GET = request.args  # django compatibility
 
     q = request.GET.get('q', 'json')
     filename = 'sample-' + datetime.now().strftime('%Y-%m-%d-%H-%M')
     try:
-        task = generate_sample_task(project)
+        task = generate_sample_task(g.project)
     except (ValueError, ValidationError, lxml.etree.Error):
         return HttpResponse('error while example generating', status=status.HTTP_400_BAD_REQUEST)
 
@@ -451,7 +445,7 @@ def api_import_example_file():
         output = pd.read_json(json.dumps(tasks), orient='records').to_csv(index=False, sep='\t')
 
     elif q == 'txt':
-        if len(project.data_types.keys()) > 1:
+        if len(g.project.data_types.keys()) > 1:
             raise ValueError('TXT is unsupported for projects with multiple sources in config')
 
         filename += '.txt'
@@ -469,7 +463,6 @@ def api_import_example_file():
     response.headers['Content-Disposition'] = 'attachment; filename=%s' % filename
     response.headers['filename'] = filename
 
-    project.analytics.send(getframeinfo(currentframe()).function)
     return response
 
 
@@ -477,8 +470,6 @@ def api_import_example_file():
 @requires_auth
 @exception_treatment
 def api_import():
-    project = project_get_or_create()
-
     # make django compatibility for uploader module
     class DjangoRequest:
         POST = request.form
@@ -489,24 +480,29 @@ def api_import():
 
     start = time.time()
     # get tasks from request
-    parsed_data = uploader.load_tasks(DjangoRequest(), project)
+    parsed_data, formats = uploader.load_tasks(DjangoRequest(), g.project)
     # validate tasks
-    validator = TaskValidator(project)
+    validator = TaskValidator(g.project)
     try:
         new_tasks = validator.to_internal_value(parsed_data)
     except ValidationError as e:
         return make_response(jsonify(e.msg_to_list()), status.HTTP_400_BAD_REQUEST)
 
     max_id_in_old_tasks = -1
-    if not project.no_tasks():
-        max_id_in_old_tasks = project.source_storage.max_id()
+    if not g.project.no_tasks():
+        max_id_in_old_tasks = g.project.source_storage.max_id()
 
     new_tasks = Tasks().from_list_of_dicts(new_tasks, max_id_in_old_tasks + 1)
-    project.source_storage.set_many(new_tasks.keys(), new_tasks.values())
+    g.project.source_storage.set_many(new_tasks.keys(), new_tasks.values())
+
+    # if tasks have completion - we need to implicitly save it to target
+    for i in new_tasks.keys():
+        for completion in new_tasks[i].get('completions', []):
+            g.project.save_completion(int(i), completion)
 
     # update schemas based on newly uploaded tasks
-    project.update_derived_input_schema()
-    project.update_derived_output_schema()
+    g.project.update_derived_input_schema()
+    g.project.update_derived_output_schema()
 
     duration = time.time() - start
     return make_response(jsonify({
@@ -514,6 +510,7 @@ def api_import():
         'completion_count': validator.completion_count,
         'prediction_count': validator.prediction_count,
         'duration': duration,
+        'formats': formats,
         'new_task_ids': [t for t in new_tasks]
     }), status.HTTP_201_CREATED)
 
@@ -523,9 +520,8 @@ def api_import():
 @exception_treatment
 def api_export():
     export_format = request.args.get('format')
-    project = project_get_or_create()
     now = datetime.now()
-    completion_dir = project.config['output_dir']
+    completion_dir = g.project.config['output_dir']
 
     project_export_dir = os.path.join(os.path.dirname(completion_dir), 'export')
     os.makedirs(project_export_dir, exist_ok=True)
@@ -533,13 +529,12 @@ def api_export():
     zip_dir = os.path.join(project_export_dir, now.strftime('%Y-%m-%d-%H-%M-%S'))
     os.makedirs(zip_dir, exist_ok=True)
 
-    project.converter.convert(completion_dir, zip_dir, format=export_format)
+    g.project.converter.convert(completion_dir, zip_dir, format=export_format)
     shutil.make_archive(zip_dir, 'zip', zip_dir)
     shutil.rmtree(zip_dir)
 
     response = send_file(zip_dir+'.zip', as_attachment=True)
     response.headers['filename'] = os.path.basename(zip_dir+'.zip')
-    project.analytics.send(getframeinfo(currentframe()).function)
     return response
 
 
@@ -549,23 +544,19 @@ def api_export():
 def api_generate_next_task():
     """ Generate next task to label
     """
-    project = project_get_or_create()
     # try to find task is not presented in completions
-    completed_tasks_ids = project.get_completions_ids()
-    task = project.next_task(completed_tasks_ids)
+    completed_tasks_ids = g.project.get_completions_ids()
+    task = g.project.next_task(completed_tasks_ids)
     if task is None:
         # no tasks found
-        project.analytics.send(getframeinfo(currentframe()).function, error=404)
         return make_response('', 404)
 
     task = resolve_task_data_uri(task)
 
-    #project.analytics.send(getframeinfo(currentframe()).function)
-
     # collect prediction from multiple ml backends
-    if project.ml_backends_connected:
-        task = project.make_predictions(task)
-    logger.debug('Next task:\n' + json.dumps(task))
+    if g.project.ml_backends_connected:
+        task = g.project.make_predictions(task)
+    logger.debug('Next task:\n' + str(task.get('id', None)))
     return make_response(jsonify(task), 200)
 
 
@@ -574,17 +565,16 @@ def api_generate_next_task():
 @exception_treatment
 def api_project():
     """ Project global operation"""
-    project = project_get_or_create(multi_session_force_recreate=False)
     code = 200
 
     if request.method == 'POST' and request.args.get('new', False):
-        project = project_get_or_create(multi_session_force_recreate=True)
+        g.project = project_get_or_create(multi_session_force_recreate=True)
         code = 201
     elif request.method == 'PATCH':
-        project.update_params(request.json)
+        g.project.update_params(request.json)
         code = 201
 
-    output = project.serialize()
+    output = g.project.serialize()
     output['multi_session_mode'] = input_args.command != 'start-multi-session'
     return make_response(jsonify(output), code)
 
@@ -593,47 +583,43 @@ def api_project():
 @requires_auth
 @exception_treatment
 def api_project_storage_settings():
-    project = project_get_or_create()
 
     # GET: return selected form, populated with current storage parameters
     if request.method == 'GET':
         # render all forms for caching in web
         all_forms = {'source': {}, 'target': {}}
         for storage_for in all_forms:
-            for name, description in project.get_available_storage_names(storage_for).items():
-                current_type = project.config.get(storage_for, {'type': ''})['type']
+            for name, description in g.project.get_available_storage_names(storage_for).items():
+                current_type = g.project.config.get(storage_for, {'type': ''})['type']
                 current = name == current_type
                 form_class = get_storage_form(name)
-                form = form_class(data=project.get_storage(storage_for).get_params()) if current else form_class()
+                form = form_class(data=g.project.get_storage(storage_for).get_params()) if current else form_class()
                 all_forms[storage_for][name] = {
                     'fields': [serialize_class(field) for field in form],
                     'type': name, 'current': current, 'description': description,
-                    'path': getattr(project, storage_for + '_storage').readable_path
+                    'path': getattr(g.project, storage_for + '_storage').readable_path
                 }
                 # generate data key automatically
-                if project.data_types.keys():
+                if g.project.data_types.keys():
                     for field in all_forms[storage_for][name]['fields']:
                         if field['name'] == 'data_key' and not field['data']:
-                            field['data'] = list(project.data_types.keys())[0]
-        project.analytics.send(getframeinfo(currentframe()).function, method=request.method)
+                            field['data'] = list(g.project.data_types.keys())[0]
         return make_response(jsonify(all_forms), 200)
 
     # POST: update storage given filled form
     if request.method == 'POST':
         selected_type = request.args.get('type', '')
         storage_for = request.args.get('storage_for')
-        current_type = project.config.get(storage_for, {'type': ''})['type']
+        current_type = g.project.config.get(storage_for, {'type': ''})['type']
         selected_type = selected_type if selected_type else current_type
 
         form = get_storage_form(selected_type)(data=request.json)
-        project.analytics.send(
-            getframeinfo(currentframe()).function, method=request.method, storage=selected_type,
-            storage_for=storage_for)
+
         if form.validate_on_submit():
             storage_kwargs = dict(form.data)
             storage_kwargs['type'] = request.json['type']  # storage type
             try:
-                project.update_storage(storage_for, storage_kwargs)
+                g.project.update_storage(storage_for, storage_kwargs)
             except Exception as e:
                 traceback = tb.format_exc()
                 logger.error(str(traceback))
@@ -645,25 +631,12 @@ def api_project_storage_settings():
             return make_response(jsonify({'errors': form.errors}), 400)
 
 
-@app.route('/api/projects/1/task_ids/', methods=['GET'])
-@requires_auth
-@exception_treatment
-def api_all_task_ids():
-    """ Get all tasks ids
-    """
-    project = project_get_or_create()
-    ids = list(sorted(project.source_storage.ids()))
-    project.analytics.send(getframeinfo(currentframe()).function)
-    return make_response(jsonify(ids), 200)
-
-
 @app.route('/api/tasks/', methods=['GET'])
 @requires_auth
 @exception_treatment
 def api_all_tasks():
     """ Get full tasks with pagination, completions and predictions
     """
-    project = project_get_or_create()
     page, page_size = int(request.args.get('page', 1)), int(request.args.get('page_size', 10))
     order = request.args.get('order', 'id')
     if page < 1 or page_size < 1:
@@ -675,9 +648,9 @@ def api_all_tasks():
         return make_response(jsonify({'detail': 'Incorrect order'}), 422)
 
     # get task ids and sort them by completed time
-    task_ids = project.source_storage.ids()
-    completed_at = project.get_completed_at()
-    skipped_status = project.get_skipped_status()
+    task_ids = g.project.source_storage.ids()
+    completed_at = g.project.get_completed_at()
+    skipped_status = g.project.get_skipped_status()
 
     # ordering
     pre_order = ({
@@ -704,10 +677,12 @@ def api_all_tasks():
     # get tasks with completions
     tasks = []
     for item in paginated:
+        if item['completed_at'] != 'undefined' and item['completed_at'] is not None:
+            item['completed_at'] = timestamp_to_local_datetime(item['completed_at']).strftime('%Y-%m-%d %H:%M:%S')
         i = item['id']
-        task = project.get_task_with_completions(i)
+        task = g.project.get_task_with_completions(i)
         if task is None:  # no completion at task
-            task = project.source_storage.get(i)
+            task = g.project.source_storage.get(i)
         else:
             task['completed_at'] = item['completed_at']
             task['has_skipped_completions'] = item['has_skipped_completions']
@@ -725,15 +700,16 @@ def api_tasks(task_id):
     """
     # try to get task with completions first
     task_id = int(task_id)
-    project = project_get_or_create()
     if request.method == 'GET':
-        task_data = project.get_task_with_completions(task_id) or project.source_storage.get(task_id)
+        task_data = g.project.get_task_with_completions(task_id) or g.project.source_storage.get(task_id)
         task_data = resolve_task_data_uri(task_data)
-        project.analytics.send(getframeinfo(currentframe()).function)
+
+        if g.project.ml_backends_connected:
+            task_data = g.project.make_predictions(task_data)
+
         return make_response(jsonify(task_data), 200)
     elif request.method == 'DELETE':
-        project.remove_task(task_id)
-        project.analytics.send(getframeinfo(currentframe()).function)
+        g.project.remove_task(task_id)
         return make_response(jsonify('Task deleted.'), 204)
 
 
@@ -743,8 +719,7 @@ def api_tasks(task_id):
 def api_tasks_delete():
     """ Delete all tasks & completions
     """
-    project = project_get_or_create()
-    project.delete_tasks()
+    g.project.delete_tasks()
     return make_response(jsonify({}), 204)
 
 
@@ -754,9 +729,7 @@ def api_tasks_delete():
 def api_all_completion_ids():
     """ Get all completion ids
     """
-    project = project_get_or_create()
-    ids = project.get_completions_ids()
-    project.analytics.send(getframeinfo(currentframe()).function)
+    ids = g.project.get_completions_ids()
     return make_response(jsonify(ids), 200)
 
 
@@ -766,19 +739,28 @@ def api_all_completion_ids():
 def api_completions(task_id):
     """ Delete or save new completion to output_dir with the same name as task_id
     """
-    project = project_get_or_create()
-
     if request.method == 'POST':
         completion = request.json
         completion.pop('state', None)  # remove editor state
         completion.pop('skipped', None)
         completion.pop('was_cancelled', None)
-        completion_id = project.save_completion(int(task_id), completion)
-        project.analytics.send(getframeinfo(currentframe()).function)
+        completion_id = g.project.save_completion(int(task_id), completion)
         return make_response(json.dumps({'id': completion_id}), 201)
+    else:
+        return make_response('Incorrect request method', 500)
+
+
+@app.route('/api/project/completions/', methods=['DELETE'])
+@requires_auth
+@exception_treatment
+def api_all_completions():
+    """ Delete all completions
+    """
+    if request.method == 'DELETE':
+        g.project.delete_all_completions()
+        return make_response('done', 201)
 
     else:
-        project.analytics.send(getframeinfo(currentframe()).function, error=500)
         return make_response('Incorrect request method', 500)
 
 
@@ -787,13 +769,11 @@ def api_completions(task_id):
 @exception_treatment
 def api_tasks_cancel(task_id):
     task_id = int(task_id)
-    project = project_get_or_create()
     skipped_completion = request.json
     skipped_completion['was_cancelled'] = True  # for platform support
     skipped_completion['skipped'] = True
 
-    completion_id = project.save_completion(task_id, skipped_completion)
-    project.analytics.send(getframeinfo(currentframe()).function)
+    completion_id = g.project.save_completion(task_id, skipped_completion)
     return make_response(json.dumps({'id': completion_id}), 201)
 
 
@@ -804,18 +784,13 @@ def api_completion_by_id(task_id, completion_id):
     """ Delete or save new completion to output_dir with the same name as task_id.
         completion_id with different IDs is not supported in this backend
     """
-    project = project_get_or_create()
-
     if request.method == 'DELETE':
-        if project.config.get('allow_delete_completions', False):
-            project.delete_completion(int(task_id))
-            project.analytics.send(getframeinfo(currentframe()).function)
+        if g.project.config.get('allow_delete_completions', False):
+            g.project.delete_completion(int(task_id))
             return make_response('deleted', 204)
         else:
-            project.analytics.send(getframeinfo(currentframe()).function, error=422)
             return make_response('Completion removing is not allowed in server config', 422)
     else:
-        project.analytics.send(getframeinfo(currentframe()).function, error=500)
         return make_response('Incorrect request method', 500)
 
 
@@ -827,15 +802,13 @@ def api_completion_update(task_id, completion_id):
         This is technical api call for editor testing only. It's used for Rewrite button in editor.
     """
     task_id = int(task_id)
-    project = project_get_or_create()
     completion = request.json
 
     completion.pop('state', None)  # remove editor state
     completion['skipped'] = completion['was_cancelled'] = False  # pop is a bad idea because of dict updating inside
 
     completion['id'] = int(completion_id)
-    project.save_completion(task_id, completion)
-    project.analytics.send(getframeinfo(currentframe()).function)
+    g.project.save_completion(task_id, completion)
     return make_response('ok', 201)
 
 
@@ -845,19 +818,15 @@ def api_completion_update(task_id, completion_id):
 def api_instruction():
     """ Instruction for annotators
     """
-    project = project_get_or_create()
-    project.analytics.send(getframeinfo(currentframe()).function)
-    return make_response(project.config['instruction'], 200)
+    return make_response(g.project.config['instruction'], 200)
 
 
 @app.route('/api/remove-ml-backend', methods=['POST'])
 @requires_auth
 @exception_treatment
 def api_remove_ml_backend():
-    project = project_get_or_create()
     ml_backend_name = request.json['name']
-    project.remove_ml_backend(ml_backend_name)
-    project.analytics.send(getframeinfo(currentframe()).function)
+    g.project.remove_ml_backend(ml_backend_name)
     return make_response(jsonify('Deleted!'), 204)
 
 
@@ -871,13 +840,12 @@ def api_predict():
         task = {'data': request.json}
     else:
         task = request.json
-    project = project_get_or_create()
-    if project.ml_backends_connected:
-        task_with_predictions = project.make_predictions(task)
-        project.analytics.send(getframeinfo(currentframe()).function)
+    if g.project.ml_backends_connected:
+        task_with_predictions = g.project.make_predictions(task)
+        g.project.analytics.send(getframeinfo(currentframe()).function)
         return make_response(jsonify(task_with_predictions), 200)
     else:
-        project.analytics.send(getframeinfo(currentframe()).function, error=400)
+        g.project.analytics.send(getframeinfo(currentframe()).function, error=400)
         return make_response(jsonify("No ML backend"), 400)
 
 
@@ -886,20 +854,16 @@ def api_predict():
 @exception_treatment
 def api_train():
     """Send train signal to ML backend"""
-    project = project_get_or_create()
-    if project.ml_backends_connected:
-        training_started = project.train()
+    if g.project.ml_backends_connected:
+        training_started = g.project.train()
         if training_started:
             logger.debug('Training started.')
-            project.analytics.send(getframeinfo(currentframe()).function, num_backends=len(project.ml_backends))
             return make_response(jsonify({'details': 'Training started'}), 200)
         else:
             logger.debug('Training failed.')
-            project.analytics.send(getframeinfo(currentframe()).function, error=400, training_started=training_started)
             return make_response(
                 jsonify('Training is not started: seems that you don\'t have any ML backend connected'), 400)
     else:
-        project.analytics.send(getframeinfo(currentframe()).function, error=400)
         return make_response(jsonify("No ML backend"), 400)
 
 
@@ -908,18 +872,15 @@ def api_train():
 @exception_treatment
 def api_predictions():
     """Send creating predictions signal to ML backend"""
-    project = project_get_or_create()
-    if project.ml_backends_connected:
+    if g.project.ml_backends_connected:
         # get tasks ids without predictions
         tasks_with_predictions = {}
-        for task_id, task in project.source_storage.items():
-            task_pred = project.make_predictions(task)
+        for task_id, task in g.project.source_storage.items():
+            task_pred = g.project.make_predictions(task)
             tasks_with_predictions[task_pred['id']] = task_pred
-        project.source_storage.set_many(tasks_with_predictions.keys(), tasks_with_predictions.values())
-
+        g.project.source_storage.set_many(tasks_with_predictions.keys(), tasks_with_predictions.values())
         return make_response(jsonify({'details': 'Predictions done.'}), 200)
     else:
-        project.analytics.send(getframeinfo(currentframe()).function, error=400)
         return make_response(jsonify("No ML backend"), 400)
 
 
@@ -942,17 +903,15 @@ def version():
 def get_data_file(filename):
     """ External resource serving
     """
-    project = project_get_or_create()
-
     # support for upload via GUI
     if filename.startswith('upload/'):
-        path = os.path.join(project.name, filename)
+        path = os.path.join(g.project.path, filename)
         directory = os.path.abspath(os.path.dirname(path))
         filename = os.path.basename(path)
         return flask.send_from_directory(directory, filename, as_attachment=True)
 
     # serving files from local storage
-    if not project.config.get('allow_serving_local_files'):
+    if not g.project.config.get('allow_serving_local_files'):
         raise FileNotFoundError('Serving local files is not allowed. '
                                 'Use "allow_serving_local_files": true config option to enable local serving')
     directory = request.args.get('d')
@@ -1019,8 +978,9 @@ def main():
         label_studio.utils.auth.PASSWORD = input_args.password or config.get('password', '')
 
         # set host name
-        host = input_args.host or config.get('host', 'localhost')
+        host = input_args.host or config.get('host', 'localhost')  # name for external links generation
         port = input_args.port or config.get('port', 8080)
+        server_host = 'localhost' if host == 'localhost' else '0.0.0.0'  # web server host
 
         # ssl certificate and key
         cert_file = input_args.cert_file or config.get('cert')
@@ -1038,29 +998,29 @@ def main():
                   '* Trying to start at ' + str(port) +
                   '\n****************\n')
 
-        set_web_protocol(config.get('protocol', 'http://'))
+        set_web_protocol(input_args.protocol or config.get('protocol', 'http://'))
         set_full_hostname(get_web_protocol() + host.replace('0.0.0.0', 'localhost') + ':' + str(port))
 
         start_browser('http://localhost:' + str(port), input_args.no_browser)
         if input_args.use_gevent:
             app.debug = input_args.debug
             ssl_args = {'keyfile': key_file, 'certfile': cert_file} if ssl_context else {}
-            http_server = WSGIServer((host, port), app, log=app.logger, **ssl_args)
+            http_server = WSGIServer((server_host, port), app, log=app.logger, **ssl_args)
             http_server.serve_forever()
         else:
-            app.run(host=host, port=port, debug=input_args.debug, ssl_context=ssl_context)
+            app.run(host=server_host, port=port, debug=input_args.debug, ssl_context=ssl_context)
 
     # On `start-multi-session` command, server creates one project per each browser sessions
     elif input_args.command == 'start-multi-session':
-        host = input_args.host or '0.0.0.0'
+        server_host = input_args.host or '0.0.0.0'
         port = input_args.port or 8080
 
         if input_args.use_gevent:
             app.debug = input_args.debug
-            http_server = WSGIServer((host, port), app, log=app.logger)
+            http_server = WSGIServer((server_host, port), app, log=app.logger)
             http_server.serve_forever()
         else:
-            app.run(host=host, port=port, debug=input_args.debug)
+            app.run(host=server_host, port=port, debug=input_args.debug)
 
 
 if __name__ == "__main__":
